/**************************************************************************/
/** TSTNTDLL.C Test program to exercise Solartron IMP Windows NT DLL	 **/
/**	       Microsoft C						 **/
/** Initial: 02-May-95 08:21						 **/
/** Latest:								 **/
/** 12-May-95 17:43 Further development					 **/
/** 17-May-95 09:10 Changed the Function names to mirror the names used  **/
/**		    in the DLL. 					 **/
/** 25-May-95 16:45 Added a terminating character to the IMP firmware	 **/
/**		    revision after it has NULLs converted into spaces.	 **/
/** 19-Jun-95 15:43 Now Version 1.1					 **/
/**		    Added LocateCard to search for the IMP Adaptor Card	 **/
/**		    from BaseAddr (Start Address) up to DFE0.		 **/
/**		    Note: If the addr command line is used eg addr ca00  **/
/**		    then the start search address is set to the value	 **/
/**		    specified.						 **/
/**		    All bytes of the Poll Table are initialised to 1	 **/
/** 20-Jun-95 11:21 Added LocateIMP to search the Poll Table for the	 **/
/**		    first IMP online.					 **/
/**		    Removed the	impaddr command line paramter - first	 **/
/**		    IMP online is now used.				 **/
/** 23-Jun-95 09:25 Amended the inspection of the return values from	 **/
/**		    Imp_Test in order to use IMP DLL Version 1.00d	 **/
/**		    Added the prototype for Imp_ValToByte		 **/
/** 26-Jun-95 14:48 Removed the definition of WinText in MainWndProc	 **/
/**		    because it is no longer used.			 **/
/** 21-Dec-95 18:16 Now Version 1.2					 **/
/**		    Added the prototype for Imp_PowerOff		 **/
/**************************************************************************/

/******************** FUNCTIONS *******************************************/
/** HexToInt -		 Convert Hex To Integer 			 **/
/** SetAdapAddr -	 Set Adaptor Card Address			 **/
/** WinMain -		 Calls InitApplication, Processes Message Loop	 **/
/** InitApplication -	 Initialises Window Data, Registers the Window	 **/
/** InitInstance -	 Saves Instance Handle, Creates Main Window	 **/
/** LocateCard -	 Searches for the IMP Adaptor Card		 **/
/** LocateIMP -		 Searches for the First IMP Online		 **/
/** TestIMPDrv -	 Test IMP Drivers				 **/
/** LoadIMPDrv -	 Load IMP Driver				 **/
/** MainWndProc -	 Processes Messages				 **/
/** About -		 Processes Messages for About... Dialog Box	 **/
/**************************************************************************/

#define  STRICT			     /** Ensure strict type checking **/
#define _fstrlen strlen

#include <windows.h>		     /** Windows Definitions **/
#include <memory.h>		     /** Memory Definitions **/
#include <dos.h>		     /** DOS Definitions **/
#include <string.h>		     /** String Definitions **/
#include <stdlib.h>		     /** Standard Library Definitions **/
#include <math.h>		     /** Math Library Definitions **/
#include "impNTdef.h"		     /** IMP Windows 3.1 General Definitions **/
#include "tstNTdll.h"		     /** Test DLL Definitions **/

HINSTANCE hInst;		     /** Handle to Current Instance **/
HINSTANCE hImpDLL;		     /** Handle to IMP DLL **/
char	  temp[80];		     /** Text **/
char	  ImpDrvOK = 0;		     /** IMP Driver OK to Use 0=No, 1=Yes **/
int	  nDrawX = 0;		     /** Num Pixels for " Across **/
int	  nDrawY = 0;		     /** Num Pixels for " Down **/
int	  CurrYPos = 0;		     /** Current Y Position (Pixels) **/
int	  nYLineFeed = 0;	     /** Num Pixels Y Line Feed **/
short	  CurHand = -1;		     /** Current IMP Adaptor Handle **/
short	  Poll_Tab[MAXPOLLTAB];	     /** IMP Poll Table **/
short	  DrvStat = 0;		     /** IMP Driver Status **/
short	  ImpNum = 1;		     /** Default IMP Address **/
short	  Imp_Sig[32];		     /** IMP Data Significance **/
float	  Imp_Data[32];		     /** IMP Data **/
unsigned  short CardAddr = 0xC800;	     /** Adaptor Card Address **/
unsigned  short BaseAddr = 0xC800;	     /** Adaptor Card Base Address **/
extern	  int  __argc;		     /** Number Command Line Parameters **/
extern	  char **__argv;	     /** Command Line Parameters **/

void (FAR PASCAL *Imp_Init) (short far*,short far*,short far*); /** Address of Imp_Init **/
void (FAR PASCAL *Imp_Tx) (short far*,char far*,short far*,short far*,short far*); /** Address of Imp_Tx **/
void (FAR PASCAL *Imp_Test) (short far*,short far*,short far*,short far*); /** Address of Imp_Test **/
void (FAR PASCAL *Imp_String) (short far*,short far*,char far*,short far*,short far*,short far*); /** Imp_String **/
void (FAR PASCAL *Imp_Numeric) (short far*,short far*,short far*,short far*,float far*,short far*,short far*); /** Imp_Numeric **/
void (FAR PASCAL *Imp_ValToByte) (float far*,short far*); /** Address of Imp_ValToByte **/
void (FAR PASCAL *Imp_Connect) (unsigned short far*,short far*); /** Address of Imp_Connect **/
void (FAR PASCAL *Imp_Disconnect) (short far*);  /** Address of Imp_Disconnect **/
short  (FAR PASCAL *Imp_CardInUse) ();  /** Address of Imp_CardInUse **/
void (FAR PASCAL *Imp_DLLVersion) (char far*); /** Address of Imp_DLLVersion **/
void (FAR PASCAL *Imp_DLLDelay) (unsigned short); /** Address of Imp_DLLDelay **/
void (FAR PASCAL *Imp_PowerOff) (short far*); /** Address of Imp_PowerOff **/

/******************** HEXTOINT ********************************************/
int  HexToInt(int Hex)		     /** Convert Hex To Integer **/
   {
    int  Ret = 0;		     /** Return Value **/

    Hex = toupper(Hex); 	     /** Convert to Upper Case **/
    switch (Hex)		     /** Select Character **/
      {
       case '0':		     /** ASCII 0 **/
       case '1':		     /** ASCII 1 **/
       case '2':		     /** ASCII 2 **/
       case '3':		     /** ASCII 3 **/
       case '4':		     /** ASCII 4 **/
       case '5':		     /** ASCII 5 **/
       case '6':		     /** ASCII 6 **/
       case '7':		     /** ASCII 7 **/
       case '8':		     /** ASCII 8 **/
       case '9':		     /** ASCII 9 **/
	  Ret = Hex - 48;	     /** Set Value **/
	  break;

       case 'A':		     /** ASCII A **/
       case 'B':		     /** ASCII B **/
       case 'C':		     /** ASCII C **/
       case 'D':		     /** ASCII D **/
       case 'E':		     /** ASCII E **/
       case 'F':		     /** ASCII F **/
	  Ret = Hex - 55;	     /** Set Value (65-55=10) **/
	  break;
      }
    return (Ret);		     /** Return Integer **/
   }
/******************** END HEXTOINT ****************************************/


/******************** SETADAPADDR *****************************************/
void SetAdapAddr(char *AddrStr)	     /** Set Adaptor Card Address **/
   {
    int  Lp = 0;		     /** Loop Controller **/
    int  Indx = 0;		     /** String Index **/

    BaseAddr = 0;		     /** Clear Base Address **/
    for (Lp=3; Lp>0; Lp--, Indx++)   /** Loop through 3 Bytes **/
       {
	BaseAddr += ((int)pow(16,Lp) * HexToInt(*(AddrStr+Indx)));
       }
    BaseAddr += HexToInt(*(AddrStr+Indx)); /** Last Byte **/
   }
/******************** END SETADAPADDR *************************************/


/******************** WINMAIN *********************************************/
int PASCAL WinMain(hinstCurrent, hinstPrevious, lpszCmdLine, nCmdShow)
HINSTANCE hinstCurrent; 	     /** Handle of current instance **/
HINSTANCE hinstPrevious;	     /** Handle of previous instance **/
LPSTR lpszCmdLine;		     /** Address of command line **/

int nCmdShow;			     /** Show window type (open/icon) **/
   {
    int CmdLp = 0;		     /** Command Line Loop Controller **/
    MSG msg;			     /** Message **/
	int   alright;


    if (!hinstPrevious) 	     /** No other instances of prog running ? **/
      {
       if (!InitApplication(hinstCurrent))  /** Initialise shared ? **/
	 return FALSE;		     /** Exit if unable to initialise **/
      }

   if (__argc > 1)		     /** Command Line Params to process ? **/
     {
      for (CmdLp=1; CmdLp<__argc; CmdLp++)  /** Process Commands **/
	 {
	  if (!stricmp(__argv[CmdLp],"addr")) /** New Card Base Address ? **/
	    {
	     SetAdapAddr(__argv[CmdLp+1]); /** Set Adaptor Card Address **/
	    }
	 }
     }

   if (!InitInstance(hinstCurrent, nCmdShow))  /** Initialise Instance Error ? **/
     return FALSE;		     /** Return Error **/

   /** Acquire and despatch messages until WM_QUIT message is received **/
   alright = GetMessage(&msg, NULL, 0, 0);

   while (alright == TRUE)
   {
       TranslateMessage(&msg);	     /** Translate virtual key codes **/
       DispatchMessage(&msg);	     /** Dispatch message to window **/

	   alright = GetMessage(&msg, NULL, 0, 0);
   }
   return (msg.wParam); 	     /** Value from PostQuitMessage **/
  }
/******************** END WINMAIN *****************************************/


/******************** INITAPPLICATION *************************************/
BOOL InitApplication(hinstCurrent)
HINSTANCE hinstCurrent; 	     /** Handle of current instance **/
  {
   WNDCLASS  wc;		     /** Window class structure **/

   wc.style = CS_HREDRAW | CS_VREDRAW;		     /** Class style(s) **/
   wc.lpfnWndProc = MainWndProc;     /** Window proc for window of this class **/
   wc.cbClsExtra = 0;		     /** No per-class extra data **/
   wc.cbWndExtra = 0;		     /** No per-window extra data **/
   wc.hInstance = hinstCurrent;	     /** Application that owns class **/
   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);	 /** Minimised window Icon **/
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);	 /** Cursor in the window **/
   wc.hbrBackground = GetStockObject(WHITE_BRUSH); /** Background brush **/
   wc.lpszMenuName = "TestDLLMenu";   /** Menu name in .RC file **/
   wc.lpszClassName = "GenericWClass";/** Name in create window **/

   return (RegisterClass(&wc));      /** Register Window Class, Return Result **/
  }
/******************** END INITAPPLICATION *********************************/


/******************** INITINSTANCE ****************************************/
BOOL InitInstance(hinstCurrent, nCmdShow)
HINSTANCE hinstCurrent; 	     /** Handle of current instance **/
int nCmdShow;			     /** Param for first show window call **/
  {
   HWND  hWnd;			     /** Handle of Main Window **/
   HMENU hMenu;			     /** Menu Handle **/

   hInst = hinstCurrent;	     /** Save the instance handle **/

   /** Create a main window for this application instance **/
   hWnd = CreateWindow(
      "GenericWClass",		     /** See Register class call **/
      "IMP DLL Test Program",	     /** Text for title bar **/
      WS_OVERLAPPEDWINDOW,	     /** Window style **/
      CW_USEDEFAULT,		     /** Default horizontal position **/
      CW_USEDEFAULT,		     /** Default vertical position **/
      CW_USEDEFAULT,		     /** Default width **/
      CW_USEDEFAULT,		     /** Default height **/
      NULL,			     /** Overlapped windows have no parent **/
      NULL,			     /** Use window class menu **/
      hinstCurrent,		     /** This instance owns this window **/
      NULL			     /** Pointer not needed **/
     );

   if (hWnd == NULL)		     /** Error Creating Window ? **/
     return (FALSE);		     /** Return Error **/

   ShowWindow(hWnd, nCmdShow);	     /** Show Window **/
   UpdateWindow(hWnd);		     /** Update Window **/

   ImpDrvOK = LoadIMPDrv(hWnd);	     /** Load IMP Driver **/
   if (!ImpDrvOK)		     /** IMP DLL Load Error ? **/
     {
      hMenu = GetMenu(hWnd);	     /** Get Menu Handle **/
      EnableMenuItem(hMenu, IDM_TESTDRV, MF_GRAYED | MF_BYCOMMAND);
     }
   return (TRUE);		     /** Return OK **/
  }
/******************** END INITINSTANCE ************************************/


/******************** LOCATECARD ******************************************/
int  LocateCard()		     /** Locate IMP Adaptor Card **/
   {
    char LpExit = 0;		     /** Loop Exit Flag **/
    int  AdapFound = 0;		     /** Adaptor Card Found 0=No 1=Yes **/
    short  InitStat = 0;		     /** Initialise Status **/

    CardAddr = BaseAddr;	     /** Set Base Address **/
    while (!LpExit)		     /** Wait for Loop Exit **/
      {
       Imp_Connect(&CardAddr,&CurHand); /** Connect to Adaptor Memory **/
       if (CurHand >= 0)	     /** Valid Card Handle ? **/
	 {
	  Imp_Init(&Poll_Tab[0],&InitStat,&CurHand); /** Initialise IMP Adaptor **/
	  if (!InitStat)	     /** Initialise OK ? **/
	    {
	     LpExit = 1;	     /** Set Loop Exit Flag **/
	     AdapFound = 1;	     /** Adaptor Card Found **/
	    }
	  else			     /** Initialise Error **/
	    {
	     Imp_Disconnect(&CurHand);/** Disconnect from Adaptor Memory **/
	     CardAddr += 0x100;	     /** Increment Card Address **/
	     if (CardAddr > 0xDFE0)  /** Past Last Address ? **/
	       LpExit = 2;	     /** Set Exit Flag **/
	    }
	 }
       else			     /** Invalid Card Handle **/
	 LpExit = 3;		     /** Set Exit Flag **/
      }
    return (AdapFound);		     /** Return Status **/
   }
/******************** END LOCATECARD **************************************/


/******************** LOCATEIMP *******************************************/
int  LocateIMP()		     /** Locate First IMP Online **/
   {
    char Lp = 0;		     /** Loop Controller **/
    char LpExit = 0;		     /** Loop Exit Flag **/
    int  ImpFound = 0;		     /** IMP Found 0=No 1=Yes **/

    ImpNum = 0;			     /** Clear IMP Address **/
    while (!LpExit)		     /** Wait for Loop Exit **/
      {
       if (Poll_Tab[Lp])	     /** IMP Online ? **/
	 {
	  ImpNum = Lp + 1;	     /** Set IMP Address **/
	  LpExit = 1;		     /** Set Loop Exit Flag **/
	  ImpFound = 1;		     /** IMP Found **/
	 }
       else			     /** Initialise Error **/
	 {
	  if (++Lp >= MAXPOLLTAB)    /** All Addresses Searched ? **/
	    LpExit = 2; 	     /** Set Exit Flag **/
	 }
      }
    return (ImpFound);		     /** Return Status **/
   }
/******************** END LOCATEIMP ***************************************/


/******************** TESTIMPDRV ******************************************/
void TestIMPDrv(HWND hWnd)	     /** Test IMP Driver **/
   {
    char WinText[80];		     /** Text to display **/
    char ImpCmd[80];		     /** Command to send to IMP **/
    char ImpStr[80];		     /** String Read from IMP **/
    char ClrLp = 0;		     /** Loop Controller **/
    short	 ImpStream = 0;		     /** IMP Stream **/
    short	 NumChars = 13;		     /** Number of Chars to Read **/
    short	 ImpChans = 20;		     /** Number of Channels to Read **/
    short	 CmdLen = 0;		     /** Length of Command String **/
    short    CardFnd = 0;		     /** IMP Adaptor Card Found 0=No 1=Yes **/
    HDC  hDC;			     /** Handle of Device Context **/
    HCURSOR hSaveCursor;	     /** Current Cursor **/
    HCURSOR hHourGlass;		     /** Hour Glass Cursor **/

    InvalidateRect(hWnd,NULL,TRUE);  /** Invalidate whole Window **/
    UpdateWindow(hWnd); 	     /** Update Window (clear) **/

    CurrYPos = nDrawY;		     /** Set Start Line **/
    hHourGlass = LoadCursor(NULL, IDC_WAIT); /** Load Hour Glass Cursor **/
    SetCapture(hWnd);		     /** Capture All Mouse Input **/
    hSaveCursor = SetCursor(hHourGlass); /** Use Hour Glass Cursor **/

    hDC = GetDC(hWnd);		     /** Get Device Context Handle **/
    wsprintf(WinText, "Searching for IMP Adaptor Card, Please Wait...");
    TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
    CurrYPos += nYLineFeed;	     /** Next Line **/
    for (ClrLp=0; ClrLp<MAXPOLLTAB; ClrLp++) /** Loop through Poll Table **/
       {
	Poll_Tab[ClrLp] = 1;	     /** Set IMP Online Flag **/
       }
    CardFnd = LocateCard();	     /** Locate IMP Adaptor Card **/
    if (CardFnd > 0)		     /** IMP Adaptor Card Found ? **/
      {
       wsprintf(WinText, "Found IMP Adaptor Card @%04X",CardAddr);
       TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
       CurrYPos += nYLineFeed;	     /** Next Line **/
       DrvStat = LocateIMP();	     /** Locate First IMP Online **/
       if (DrvStat)		     /** Any IMPs Online ? **/
	 {
	  wsprintf(WinText, "Found IMP at Address %d",ImpNum);
	  TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
	  CurrYPos += nYLineFeed;    /** Next Line **/
	  CmdLen = wsprintf(ImpCmd, "RE;SE;ST;TR");
	  Imp_Tx(&ImpNum,&ImpCmd[0],&DrvStat,&CmdLen,&CurHand); /** Transmit String to IMP **/
	  wsprintf(WinText, "IMPTX Sent Command=%s, Len=%d, Stat=%d,",(LPSTR)ImpCmd,CmdLen,DrvStat);
	  TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
	  CurrYPos += nYLineFeed;    /** Next Line **/

	  if (!DrvStat)		     /** Command Transmitted OK ? **/
	    {
	     Imp_DLLDelay(300);	     /** Delay 3 Seconds **/

	     ImpStream = 3;	     /** Steam 3 ASCII Data **/
	     Imp_Test(&ImpNum,&ImpStream,&DrvStat,&CurHand);	/** Test IMP Stream for Data **/
	     wsprintf(WinText, "IMPTEST Stream 3 Return Stat=%d,",DrvStat);
	     TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
	     CurrYPos += nYLineFeed; /** Next Line **/

	     if (DrvStat == 1)	     /** Data Ready ? **/
	       {
		NumChars = 13;	     /** Number of Chars to Read **/
		Imp_String(&ImpNum,&ImpStream,&ImpStr[0],&NumChars,&DrvStat,&CurHand);
		for (ClrLp=0; ClrLp<12; ClrLp++) /** Loop Through String **/
		   {
		    if (ImpStr[ClrLp] == '\0')	/** NULL ? **/
		      ImpStr[ClrLp] = ' '; /** Replace with space **/
		   }
		ImpStr[ClrLp] = '\0';/** Terminate String **/
		wsprintf(WinText, "IMPSTRING Stat=%d, Str=%s,",DrvStat,(LPSTR)ImpStr);
		TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
		CurrYPos += nYLineFeed; /** Next Line **/
	       }

	     ImpStream = 0;	     /** Stream 0 Numeric Data **/
	     Imp_Test(&ImpNum,&ImpStream,&DrvStat,&CurHand);	/** Test IMP Stream for Data **/
	     wsprintf(WinText, "IMPTEST Stream 0 Return Stat=%d,",DrvStat);
	     TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
	     CurrYPos += nYLineFeed; /** Next Line **/

	     if (DrvStat == 1)	     /** Data Ready ? **/
	       {
		ImpChans = 20;	     /** Read all 20 Channels **/
		Imp_Numeric(&ImpNum,&ImpStream,&ImpChans,&DrvStat,&Imp_Data[0],&Imp_Sig[0],&CurHand);
		wsprintf(WinText, "IMPNUMERIC Stat=%d,",DrvStat);
		TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
		CurrYPos += nYLineFeed; /** Next Line **/
		if (Imp_Sig[0] < 128)/** Chan 1 Data OK ? **/
		  {
		   _gcvt(Imp_Data[0],7,&ImpStr[0]);
		   wsprintf(WinText, "Chan1=%s, Sig=%d,",(LPSTR)ImpStr,Imp_Sig[0]);
		  }
		else		     /** Chan 1 Data Error **/
		  wsprintf(WinText, "Chan1=ERROR, Sig=%d,",Imp_Sig[0]);
		TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
		CurrYPos += nYLineFeed; /** Next Line **/

		if (Imp_Sig[19] < 128)	/** Chan 20 Data OK ? **/
		  {
		   _gcvt(Imp_Data[19],7,&ImpStr[0]);
		   wsprintf(WinText, "Chan20=%s, Sig=%d,",(LPSTR)ImpStr,Imp_Sig[19]);
		  }
		else		     /** Chan 20 Data Error **/
		  wsprintf(WinText, "Chan20=ERROR, Sig=%d,",Imp_Sig[19]);
		TextOut (hDC, nDrawX, CurrYPos, WinText, _fstrlen (WinText));
		CurrYPos += nYLineFeed; /** Next Line **/
	       }
	    }
	 }
       else			     /** No IMPs Online **/
	 {
	  wsprintf(temp, "ERROR - No IMPs Online");
	  TextOut (hDC, nDrawX, CurrYPos, temp, _fstrlen (temp));
	  CurrYPos += nYLineFeed;    /** Next Line **/
	 }
	   Imp_PowerOff(&CurHand);
       Imp_Disconnect(&CurHand);     /** Disconnect from Adaptor Memory **/
      }
    else			     /** Could Not Locate Adaptor Card **/
      {
       wsprintf(temp, "ERROR - Could Not Locate IMP Adaptor Card");
       TextOut (hDC, nDrawX, CurrYPos, temp, _fstrlen (temp));
       CurrYPos += nYLineFeed;	     /** Next Line **/
      }

    Imp_DLLVersion(&ImpStr[0]);	     /** Get DLL Version **/
    wsprintf(temp, "DLL Version = %s",(LPSTR)ImpStr);
    TextOut(hDC, nDrawX, CurrYPos, temp, _fstrlen(temp));
    CurrYPos += nYLineFeed;	     /** Next Line **/

    ReleaseDC(hWnd, hDC);	     /** Release Device Context **/
    SetCursor(hSaveCursor);	     /** Restore Previous Cursor **/
    ReleaseCapture();		     /** Release Mouse Input Capture **/
   }
/******************** END TESTIMPDRV **************************************/


/******************** LOADIMPDRV ******************************************/
int  LoadIMPDrv(HWND hWnd)	     /** Load IMP Driver **/
   {
    char  MsgText[80];		     /** Error Message Box Text **/
    char  MsgTitle[80];		     /** Error Message Box Title **/
    int   LoadStat = 0;		     /** DLL Load Status 1=OK **/
    int   MsgStat = 0;		     /** Error Message Box Status **/
    UINT  OldErrMode = 0;	     /** Old Error Mode **/

    OldErrMode = SetErrorMode(SEM_NOOPENFILEERRORBOX);	/** Set Error Mode **/
    hImpDLL = LoadLibrary("IMPDRVR.DLL");
    if (hImpDLL != NULL)   /** Loaded OK ? **/
      {
       (FARPROC) Imp_Init = GetProcAddress(hImpDLL,"Imp_Init");
       if (Imp_Init != NULL)	     /** Address Valid ? **/
	 {
	  (FARPROC) Imp_Tx = GetProcAddress(hImpDLL,"Imp_Tx");
	  if (Imp_Tx != NULL)	     /** Address Valid ? **/
	    {
	     (FARPROC) Imp_Test = GetProcAddress(hImpDLL,"Imp_Test");
	     if (Imp_Test != NULL)   /** Address Valid ? **/
	       {
		(FARPROC) Imp_String = GetProcAddress(hImpDLL,"Imp_String");
		if (Imp_String != NULL) /** Address Valid ? **/
		  {
		   (FARPROC) Imp_Numeric = GetProcAddress(hImpDLL,"Imp_Numeric");
		   if (Imp_Numeric != NULL) /** Address Valid ? **/
		     {
		      (FARPROC) Imp_ValToByte = GetProcAddress(hImpDLL,"Imp_ValToByte");
		      if (Imp_ValToByte != NULL) /** Address Valid ? **/
			{
			 (FARPROC) Imp_Connect = GetProcAddress(hImpDLL,"Imp_Connect");
			 if (Imp_Connect != NULL) /** Address Valid ? **/
			   {
			    (FARPROC) Imp_Disconnect = GetProcAddress(hImpDLL,"Imp_Disconnect");
			    if (Imp_Disconnect != NULL) /** Address Valid ? **/
			      {
			       (FARPROC) Imp_CardInUse = GetProcAddress(hImpDLL,"Imp_CardInUse");
			       if (Imp_CardInUse != NULL) /** Address Valid ? **/
				 {
				  (FARPROC) Imp_DLLVersion = GetProcAddress(hImpDLL,"Imp_DLLVersion");
				  if (Imp_DLLVersion != NULL) /** Address Valid ? **/
				    {
				     (FARPROC) Imp_DLLDelay = GetProcAddress(hImpDLL,"Imp_DLLDelay");
				     if (Imp_DLLDelay != NULL) /** Address Valid ? **/
				       {
					(FARPROC) Imp_PowerOff = GetProcAddress(hImpDLL,"Imp_PowerOff");
					if (Imp_PowerOff != NULL) /** Address Valid ? **/
					  {
					   LoadStat = 1;	/** DLL Loaded OK **/
					  }
				       }
				    }
				 }
			      }
			   }
			}
		     }
		  }
	       }
	    }
	 }
       if (!LoadStat)		     /** IMP DLL Initialise Error **/
	 {
	  wsprintf(MsgText, "Could Not Initialise IMPDRVR.DLL");
	  wsprintf(MsgTitle, "Test IMP DLL ERROR");
	  MsgStat = MessageBox(hWnd,(LPSTR)MsgText,(LPSTR)MsgTitle,MB_OK | MB_ICONEXCLAMATION);
	 }
      }
    else			     /** IMP DLL Load Error **/
      {
       wsprintf(MsgText, "Could Not Load IMPDRVR.DLL");
       wsprintf(MsgTitle, "Test IMP DLL ERROR");
       MsgStat = MessageBox(hWnd,(LPSTR)MsgText,(LPSTR)MsgTitle,MB_OK | MB_ICONEXCLAMATION);
      }
    OldErrMode = SetErrorMode(OldErrMode); /** Restore Error Mode **/
    return (LoadStat);		     /** Return Load Status **/
   }
/******************** END LOADIMPDRV **************************************/


/******************** MAINWNDPROC *****************************************/
LRESULT FAR PASCAL MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;			     /** Window handle **/
UINT message;			     /** Type of message **/
WPARAM wParam;			     /** Additional information **/
LPARAM lParam;			     /** Additional information **/
  {
   FARPROC lpProcAbout;		     /** Pointer to the About function **/
   HDC hDC;			     /** Display-context variable **/
   PAINTSTRUCT ps;		     /** Paint structure **/

	  switch (message)		     /** Select message **/
     {
      case WM_COMMAND:		     /** Command from a menu **/
	 switch (wParam)	     /** Select Command **/
	   {
	    case IDM_ABOUT:	     /** About... Selected **/
	       lpProcAbout = MakeProcInstance((FARPROC) About, hInst);
	       DialogBox(hInst,	     /** Handle of current instance **/
		  "AboutBox",	     /** Resource to use **/
		  hWnd,		     /** Parent handle **/
		  (DLGPROC) lpProcAbout);/** Instance address **/
	       FreeProcInstance(lpProcAbout);
	       break;

	    case IDM_TESTDRV:	     /** Test IMP Driver **/
	       if (ImpDrvOK)	     /** IMP Driver OK ? **/
		  TestIMPDrv(hWnd);  /** Test IMP Drivers **/
	       break;

	    case IDM_EXIT:	     /** Exit **/
	       DestroyWindow (hWnd); /** Destroy Window **/
	       break;
	   }
	 break;

      case WM_PAINT:		     /** Repaint screen **/
	 {
	  TEXTMETRIC tm;	     /** Text Metric **/

	  InvalidateRect(hWnd, NULL, TRUE); /** Invalidate Whole Window **/
	  hDC = BeginPaint (hWnd, &ps); /** Get display context/begin paint **/
	  GetTextMetrics (hDC, &tm); /** Get Text Metrics **/
	  nDrawX = GetDeviceCaps (hDC, LOGPIXELSX) / 4;   /** 1/4" Across **/
	  nDrawY = GetDeviceCaps (hDC, LOGPIXELSY) / 8;   /** 1/8" Down **/
	  CurrYPos = nDrawY;	     /** Set Y Position **/
	  nYLineFeed = tm.tmExternalLeading + tm.tmHeight;
	  EndPaint (hWnd,&ps);	     /** Tell Windows Painting is Complete **/
	 }
      break;

      case WM_DESTROY:		     /** Window being destroyed **/
	 if (hImpDLL != NULL)  /** Loaded OK ? **/
	   {
	    if (!Imp_CardInUse())    /** IMP Card No Longer in Use ? **/
			FreeLibrary(hImpDLL); /** Free IMP DLL **/
	   }

	 PostQuitMessage(0);	     /** Post Application Terminate Msg **/
	 break;

      default:			     /** Pass it on Unprocessed **/
	 return (DefWindowProc(hWnd, message, wParam, lParam));
     }
   return 0;
  }
/******************** END MAINWNDPROC *************************************/


/******************** ABOUT ***********************************************/
BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;			     /** Handle of Dialog box window **/
WORD message;			     /** Type of message **/
WPARAM wParam;			     /** Message specific information **/
LPARAM lParam;			     /** Message specific information **/
  {
   switch (message)		     /** Select message **/
     {
      case WM_INITDIALOG:	     /** Initialise dialog box **/
	 return TRUE;

      case WM_COMMAND:		     /** Command **/
	 if (wParam==IDOK || wParam==IDCANCEL)	/** OK or close	selected **/
	   {
	    EndDialog(hDlg, TRUE);   /** Exit dialog box **/
	    return TRUE;
	   }
	 break;
     }
   return FALSE;
  }
/******************** END ABOUT *******************************************/
